@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Author: Chris Toshok (toshok@ximian.com)
  */
@@ -34,7 +34,7 @@
 
 #define CRLF "\r\n"
 
-/** Encoding used in v-card 
+/** Encoding used in v-card
  *  Note: v-card spec defines additional 7BIT 8BIT and X- encoding
  */
 typedef enum {
@@ -64,12 +64,6 @@ struct _EVCardAttributeParam {
 
 static GObjectClass *parent_class;
 
-static void   _evc_base64_init(void);
-static size_t _evc_base64_encode_step(unsigned char *in, size_t len, gboolean break_lines, unsigned char *out, int *state, int *save);
-static size_t _evc_base64_decode_step(unsigned char *in, size_t len, unsigned char *out, int *state, unsigned int *save);
-size_t _evc_base64_decode_simple (char *data, size_t len);
-char  *_evc_base64_encode_simple (const char *data, size_t len);
-
 static void
 e_vcard_dispose (GObject *object)
 {
@@ -98,8 +92,6 @@ e_vcard_class_init (EVCardClass *klass)
 	parent_class = g_type_class_ref (G_TYPE_OBJECT);
 
 	object_class->dispose = e_vcard_dispose;
-
-	_evc_base64_init();
 }
 
 static void
@@ -135,14 +127,14 @@ e_vcard_get_type (void)
 
 
 /*  Skip newline characters and return the next character.
- *  This function takes care of folding lines, skipping 
+ *  This function takes care of folding lines, skipping
  *  newline characters if found, taking care of equal characters
  *  and other strange things.
  */
 static char*
 skip_newline (char *str, gboolean quoted_printable)
 {
-	char *p; 
+	char *p;
 	char *next;
 	char *next2;
 	p = str;
@@ -239,7 +231,7 @@ read_attribute_value (EVCardAttribute *attr, char **p, gboolean quoted_printable
 	/* read in the value */
 	str = g_string_new ("");
 	for( lp =  skip_newline( *p, quoted_printable );
-	     *lp != '\n' && *lp != '\r' && *lp != '\0'; 
+	     *lp != '\n' && *lp != '\r' && *lp != '\0';
 	     lp = skip_newline( lp, quoted_printable ) ) {
 
 		if (*lp == '=' && quoted_printable) {
@@ -254,15 +246,15 @@ read_attribute_value (EVCardAttribute *attr, char **p, gboolean quoted_printable
 
 				c = (((a>='a'?a-'a'+10:a-'0')&0x0f) << 4)
 					| ((b>='a'?b-'a'+10:b-'0')&0x0f);
-				
+
 				g_string_append_c (str, c); /* add decoded byte (this is not a unicode yet) */
 			}
-			else 
+			else
 				{
 					g_string_append_c (str, a);
 					g_string_append_c (str, b);
 				}
-			
+
 			lp++;
 
 		} else if (*lp == '\\') {
@@ -273,6 +265,12 @@ read_attribute_value (EVCardAttribute *attr, char **p, gboolean quoted_printable
 				g_string_append_c (str, '\\');
 				break;
 			}
+
+			/* beware, there might be a line break due to folding,
+			 * need next real character
+			 */
+			lp = skip_newline (lp, quoted_printable);
+
 			switch (*lp) {
 			case 'n': g_string_append_c (str, '\n'); break;
 			case 'N': g_string_append_c (str, '\n'); break;
@@ -320,7 +318,7 @@ read_attribute_params (EVCardAttribute *attr, char **p, gboolean *quoted_printab
 
 	str = g_string_new ("");
 	for( lp =  skip_newline( *p, *quoted_printable );
-	     *lp != '\n' && *lp != '\r' && *lp != '\0'; 
+	     *lp != '\n' && *lp != '\r' && *lp != '\0';
 	     lp = skip_newline( lp, *quoted_printable ) ) {
 
 		if (*lp == '"') {
@@ -351,7 +349,7 @@ read_attribute_params (EVCardAttribute *attr, char **p, gboolean *quoted_printab
 				} else if (*lp == ':') {
 					/* do nothing */
 
-				} else {				
+				} else {
 					skip_to_next_line( &lp );
 					break;
 				}
@@ -423,7 +421,7 @@ read_attribute_params (EVCardAttribute *attr, char **p, gboolean *quoted_printab
 				else {
 					/* we've got an attribute with a truly empty
 					   attribute parameter.  So it's of the form:
-					   
+
 					   ATTR;[PARAM=value;]*;[PARAM=value;]*:
 
 					   (note the extra ';')
@@ -448,7 +446,7 @@ read_attribute_params (EVCardAttribute *attr, char **p, gboolean *quoted_printab
 			g_warning ("invalid character found in parameter spec");
 			g_string_assign (str, "");
 			/*			skip_until (&lp, ":;"); */
-			
+
 			skip_to_next_line( &lp );
 		}
 	}
@@ -474,7 +472,7 @@ read_attribute (char **p)
 	/* first read in the group/name */
 	str = g_string_new ("");
 	for( lp =  skip_newline( *p, is_qp );
-	     *lp != '\n' && *lp != '\r' && *lp != '\0'; 
+	     *lp != '\n' && *lp != '\r' && *lp != '\0';
 	     lp = skip_newline( lp, is_qp ) ) {
 
 		if (*lp == ':' || *lp == ';') {
@@ -564,34 +562,34 @@ make_valid_utf8 (const gchar *name)
 	GString *string;
 	const gchar *remainder, *invalid;
 	gint remaining_bytes, valid_bytes;
-  
+
 	string = NULL;
   	remainder = name;
 	remaining_bytes = strlen (name);
-  
+
  	while (remaining_bytes != 0) {
-		if (g_utf8_validate (remainder, remaining_bytes, &invalid)) 
+		if (g_utf8_validate (remainder, remaining_bytes, &invalid))
 			break;
 	      valid_bytes = invalid - remainder;
-    
-		if (string == NULL) 
+
+		if (string == NULL)
 			string = g_string_sized_new (remaining_bytes);
 
 		g_string_append_len (string, remainder, valid_bytes);
 	        /* append U+FFFD REPLACEMENT CHARACTER */
 	        g_string_append (string, "\357\277\275");
-      
+
 	        remaining_bytes -= valid_bytes + 1;
 	        remainder = invalid + 1;
 	}
-  
+
   	if (string == NULL)
     		return g_strdup (name);
-  
+
 	g_string_append (string, remainder);
 
         g_assert (g_utf8_validate (string->str, -1, NULL));
-  
+
         return g_string_free (string, FALSE);
 }
 
@@ -605,7 +603,7 @@ parse (EVCard *evc, const char *str)
 	char *buf ;
 	char *p;
 	EVCardAttribute *attr;
-	
+
 	buf = make_valid_utf8 (str);
 
 	//buf = fold_lines (buf);
@@ -621,18 +619,22 @@ parse (EVCard *evc, const char *str)
 	if (!attr || attr->group || g_ascii_strcasecmp (attr->name, "begin")) {
 		g_warning ("vcard began without a BEGIN:VCARD\n");
 	}
-	if (attr && !g_ascii_strcasecmp (attr->name, "begin"))
+	if (attr && !g_ascii_strcasecmp (attr->name, "begin")) {
 		e_vcard_attribute_free (attr);
-	else if (attr)
+		attr = NULL;
+	} else if (attr)
 		e_vcard_add_attribute (evc, attr);
 
 	while (*p) {
 		EVCardAttribute *next_attr = read_attribute (&p);
 
 		if (next_attr) {
+			attr = next_attr;
+
 			if (g_ascii_strcasecmp (next_attr->name, "end"))
 				e_vcard_add_attribute (evc, next_attr);
-			attr = next_attr;
+			else
+				break;
 		}
 	}
 
@@ -749,7 +751,7 @@ e_vcard_construct (EVCard *evc, const char *str)
 
 /**
  * e_vcard_new:
- * 
+ *
  * Creates a new, blank #EVCard.
  *
  * Return value: A new, blank #EVCard.
@@ -1062,7 +1064,7 @@ e_vcard_attribute_copy (EVCardAttribute *attr)
  * @attr_group: group name of attributes to be removed
  * @attr_name: name of the arributes to be removed
  *
- * Removes all the attributes with group name and attribute name equal to 
+ * Removes all the attributes with group name and attribute name equal to
  * passed in values. If @attr_group is %NULL or an empty string,
  * it removes all the attributes with passed in name irrespective of
  * their group names.
@@ -1153,7 +1155,7 @@ e_vcard_add_attribute_with_value (EVCard *evcard,
  * e_vcard_add_attribute_with_values:
  * @evcard: an @EVCard
  * @attr: an #EVCardAttribute to add
- * @...: a %NULL-terminated list of values to assign to the attribute
+ * @Varargs: a %NULL-terminated list of values to assign to the attribute
  *
  * Adds @attr to @evcard, assigning the list of values to it.
  **/
@@ -1211,7 +1213,7 @@ e_vcard_attribute_add_value_decoded (EVCardAttribute *attr, const char *value, i
 		g_warning ("can't add_value_decoded with an attribute using RAW encoding.  you must set the ENCODING parameter first");
 		break;
 	case EVC_ENCODING_BASE64: {
-		char *b64_data = _evc_base64_encode_simple (value, len);
+		char *b64_data = g_base64_encode (value, len);
 		GString *decoded = g_string_new_len (value, len);
 
 		/* make sure the decoded list is up to date */
@@ -1300,10 +1302,39 @@ e_vcard_attribute_remove_value (EVCardAttribute *attr, const char *s)
 	if (l == NULL) {
 		return;
 	}
-	
+
 	attr->values = g_list_delete_link (attr->values, l);
 }
 
+/**
+ * e_vcard_attribute_remove_param:
+ * @attr: an #EVCardAttribute
+ * @param_name: a parameter name
+ *
+ * Removes the parameter @param_name from the attribute @attr.
+ *
+ * Since 1.11.3.
+ */
+void
+e_vcard_attribute_remove_param (EVCardAttribute *attr, const char *param_name)
+{
+	GList *l;
+	EVCardAttributeParam *param;
+
+	g_return_if_fail (attr != NULL);
+	g_return_if_fail (param_name != NULL);
+
+	for (l = attr->params; l; l = l->next) {
+		param = l->data;
+		if (g_ascii_strcasecmp (e_vcard_attribute_param_get_name (param),
+					param_name) == 0) {
+			attr->params = g_list_delete_link (attr->params, l);
+			e_vcard_attribute_param_free(param);
+			break;
+		}
+	}
+}
+
 /**
  * e_vcard_attribute_remove_params:
  * @attr: an #EVCardAttribute
@@ -1391,21 +1422,73 @@ e_vcard_attribute_param_copy (EVCardAttributeParam *param)
  * @param: an #EVCardAttributeParam to add
  *
  * Adds @param to @attr's list of parameters.
+ * It tests for duplicities, only new parameters are added,
+ * when a new parameter already exists in attr, then those
+ * values are merged, also without creating duplicities.
+ * When we will not add whole param, then it's freed here.
  **/
 void
 e_vcard_attribute_add_param (EVCardAttribute *attr,
 			     EVCardAttributeParam *param)
 {
+	gboolean contains;
+	GList *params, *p;
+	const char *par_name;
+
 	g_return_if_fail (attr != NULL);
 	g_return_if_fail (param != NULL);
 
-	attr->params = g_list_prepend (attr->params, param);
+	contains = FALSE;
+	params = attr->params;
+	par_name = param->name;
+
+	for (p = params; p; p = p->next) {
+		EVCardAttributeParam *param2 = p->data;
+		if (g_ascii_strcasecmp (param2->name, par_name) == 0) {
+			/* param2 has same name as our new param;
+			   better merge them than have more parameters
+			   with same name within one attribute.
+			*/
+			GList *vals,*v;
+
+			vals = param->values;
+
+			for (v = vals; v; v = v->next) {
+				const char *my_value;
+				GList *vals2,*v2;
+
+				my_value = (const char *)v->data;
+				vals2 = param2->values;
+
+				for (v2 = vals2; v2; v2 = v2->next) {
+					if (g_ascii_strcasecmp ((const char *)v2->data, my_value) == 0) {
+						break;
+					}
+				}
+
+				if (!v2) {
+					/* we did loop through all values and none of them was my_value */
+					e_vcard_attribute_param_add_value (param2, my_value);
+				}
+			}
+
+			contains = TRUE;
+			break;
+		}
+	}
+
+	if (!contains) {
+		attr->params = g_list_prepend (attr->params, param);
+	}
 
 	/* we handle our special encoding stuff here */
 
 	if (!g_ascii_strcasecmp (param->name, EVC_ENCODING)) {
 		if (attr->encoding_set) {
 			g_warning ("ENCODING specified twice");
+			if (contains) {
+				e_vcard_attribute_param_free (param);
+			}
 			return;
 		}
 
@@ -1426,6 +1509,10 @@ e_vcard_attribute_add_param (EVCardAttribute *attr,
 			g_warning ("ENCODING parameter added with no value");
 		}
 	}
+
+	if (contains) {
+		e_vcard_attribute_param_free (param);
+	}
 }
 
 /**
@@ -1562,7 +1649,7 @@ e_vcard_attribute_remove_param_value (EVCardAttribute *attr, const char *param_n
 			if (l == NULL) {
 				return;
 			}
-			
+
 			param->values = g_list_delete_link (param->values, l);
 			if (param->values == NULL) {
 				e_vcard_attribute_param_free (param);
@@ -1614,7 +1701,7 @@ e_vcard_get_attribute (EVCard     *evc,
         attrs = e_vcard_get_attributes (evc);
         for (l = attrs; l; l = l->next) {
                 EVCardAttribute *attr;
-		
+
                 attr = (EVCardAttribute *) l->data;
                 if (strcmp (attr->name, name) == 0)
                         return attr;
@@ -1660,7 +1747,7 @@ e_vcard_attribute_get_name (EVCardAttribute *attr)
  *
  * Gets the list of values from @attr. The list and its
  * contents are owned by @attr, and must not be freed.
- * 
+ *
  * Return value: A list of string values.
  **/
 GList*
@@ -1696,8 +1783,10 @@ e_vcard_attribute_get_values_decoded (EVCardAttribute *attr)
 			break;
 		case EVC_ENCODING_BASE64:
 			for (l = attr->values; l; l = l->next) {
-				char *decoded = g_strdup ((char*)l->data);
-				int len = _evc_base64_decode_simple (decoded, strlen (decoded));
+				guchar *decoded;
+				gsize len = 0;
+
+				decoded = g_base64_decode (l->data, &len);
 				attr->decoded_values = g_list_prepend (attr->decoded_values, g_string_new_len (decoded, len));
 				g_free (decoded);
 			}
@@ -1825,7 +1914,7 @@ e_vcard_attribute_has_type (EVCardAttribute *attr, const char *typestr)
 /**
  * e_vcard_attribute_get_params:
  * @attr: an #EVCardAttribute
- * 
+ *
  * Gets the list of parameters from @attr. The list and its
  * contents are owned by @attr, and must not be freed.
  *
@@ -1843,7 +1932,7 @@ e_vcard_attribute_get_params (EVCardAttribute *attr)
  * e_vcard_attribute_get_param:
  * @attr: an #EVCardAttribute
  * @name: a parameter name
- * 
+ *
  * Gets the list of values for the paramater @name from @attr. The list and its
  * contents are owned by @attr, and must not be freed.
  *
@@ -1853,10 +1942,10 @@ GList *
 e_vcard_attribute_get_param (EVCardAttribute *attr, const char *name)
 {
 	GList *params, *p;
-	
+
 	g_return_val_if_fail (attr != NULL, NULL);
 	g_return_val_if_fail (name != NULL, NULL);
-	
+
 	params = e_vcard_attribute_get_params (attr);
 
 	for (p = params; p; p = p->next) {
@@ -1888,7 +1977,7 @@ e_vcard_attribute_param_get_name (EVCardAttributeParam *param)
 /**
  * e_vcard_attribute_param_get_values:
  * @param: an #EVCardAttributeParam
- * 
+ *
  * Gets the list of values from @param. The list and its
  * contents are owned by @param, and must not be freed.
  *
@@ -1901,233 +1990,3 @@ e_vcard_attribute_param_get_values (EVCardAttributeParam *param)
 
 	return param->values;
 }
-
-
-
-/* encoding/decoding stuff ripped from camel-mime-utils.c */
-
-static char *_evc_base64_alphabet =
-"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-
-static unsigned char _evc_base64_rank[256];
-
-static void
-_evc_base64_init(void)
-{
-	int i;
-
-	memset(_evc_base64_rank, 0xff, sizeof(_evc_base64_rank));
-	for (i=0;i<64;i++) {
-		_evc_base64_rank[(unsigned int)_evc_base64_alphabet[i]] = i;
-	}
-	_evc_base64_rank['='] = 0;
-}
-
-/* call this when finished encoding everything, to
-   flush off the last little bit */
-static size_t
-_evc_base64_encode_close(unsigned char *in, size_t inlen, gboolean break_lines, unsigned char *out, int *state, int *save)
-{
-	int c1, c2;
-	unsigned char *outptr = out;
-
-	if (inlen>0)
-		outptr += _evc_base64_encode_step(in, inlen, break_lines, outptr, state, save);
-
-	c1 = ((unsigned char *)save)[1];
-	c2 = ((unsigned char *)save)[2];
-	
-	d(printf("mode = %d\nc1 = %c\nc2 = %c\n",
-		 (int)((char *)save)[0],
-		 (int)((char *)save)[1],
-		 (int)((char *)save)[2]));
-
-	switch (((char *)save)[0]) {
-	case 2:
-		outptr[2] = _evc_base64_alphabet[ ( (c2 &0x0f) << 2 ) ];
-		g_assert(outptr[2] != 0);
-		goto skip;
-	case 1:
-		outptr[2] = '=';
-	skip:
-		outptr[0] = _evc_base64_alphabet[ c1 >> 2 ];
-		outptr[1] = _evc_base64_alphabet[ c2 >> 4 | ( (c1&0x3) << 4 )];
-		outptr[3] = '=';
-		outptr += 4;
-		break;
-	}
-	if (break_lines)
-		*outptr++ = '\n';
-
-	*save = 0;
-	*state = 0;
-
-	return outptr-out;
-}
-
-/*
-  performs an 'encode step', only encodes blocks of 3 characters to the
-  output at a time, saves left-over state in state and save (initialise to
-  0 on first invocation).
-*/
-static size_t
-_evc_base64_encode_step(unsigned char *in, size_t len, gboolean break_lines, unsigned char *out, int *state, int *save)
-{
-	register unsigned char *inptr, *outptr;
-
-	if (len<=0)
-		return 0;
-
-	inptr = in;
-	outptr = out;
-
-	d(printf("we have %d chars, and %d saved chars\n", len, ((char *)save)[0]));
-
-	if (len + ((char *)save)[0] > 2) {
-		unsigned char *inend = in+len-2;
-		register int c1, c2, c3;
-		register int already;
-
-		already = *state;
-
-		switch (((char *)save)[0]) {
-		case 1:	c1 = ((unsigned char *)save)[1]; goto skip1;
-		case 2:	c1 = ((unsigned char *)save)[1];
-			c2 = ((unsigned char *)save)[2]; goto skip2;
-		}
-		
-		/* yes, we jump into the loop, no i'm not going to change it, it's beautiful! */
-		while (inptr < inend) {
-			c1 = *inptr++;
-		skip1:
-			c2 = *inptr++;
-		skip2:
-			c3 = *inptr++;
-			*outptr++ = _evc_base64_alphabet[ c1 >> 2 ];
-			*outptr++ = _evc_base64_alphabet[ c2 >> 4 | ( (c1&0x3) << 4 ) ];
-			*outptr++ = _evc_base64_alphabet[ ( (c2 &0x0f) << 2 ) | (c3 >> 6) ];
-			*outptr++ = _evc_base64_alphabet[ c3 & 0x3f ];
-			/* this is a bit ugly ... */
-			if (break_lines && (++already)>=19) {
-				*outptr++='\n';
-				already = 0;
-			}
-		}
-
-		((char *)save)[0] = 0;
-		len = 2-(inptr-inend);
-		*state = already;
-	}
-
-	d(printf("state = %d, len = %d\n",
-		 (int)((char *)save)[0],
-		 len));
-
-	if (len>0) {
-		register char *saveout;
-
-		/* points to the slot for the next char to save */
-		saveout = & (((char *)save)[1]) + ((char *)save)[0];
-
-		/* len can only be 0 1 or 2 */
-		switch(len) {
-		case 2:	*saveout++ = *inptr++;
-		case 1:	*saveout++ = *inptr++;
-		}
-		((char *)save)[0]+=len;
-	}
-
-	d(printf("mode = %d\nc1 = %c\nc2 = %c\n",
-		 (int)((char *)save)[0],
-		 (int)((char *)save)[1],
-		 (int)((char *)save)[2]));
-
-	return outptr-out;
-}
-
-
-/**
- * base64_decode_step: decode a chunk of base64 encoded data
- * @in: input stream
- * @len: max length of data to decode
- * @out: output stream
- * @state: holds the number of bits that are stored in @save
- * @save: leftover bits that have not yet been decoded
- *
- * Decodes a chunk of base64 encoded data
- **/
-static size_t
-_evc_base64_decode_step(unsigned char *in, size_t len, unsigned char *out, int *state, unsigned int *save)
-{
-	register unsigned char *inptr, *outptr;
-	unsigned char *inend, c;
-	register unsigned int v;
-	int i;
-
-	inend = in+len;
-	outptr = out;
-
-	/* convert 4 base64 bytes to 3 normal bytes */
-	v=*save;
-	i=*state;
-	inptr = in;
-	while (inptr<inend) {
-		c = _evc_base64_rank[*inptr++];
-		if (c != 0xff) {
-			v = (v<<6) | c;
-			i++;
-			if (i==4) {
-				*outptr++ = v>>16;
-				*outptr++ = v>>8;
-				*outptr++ = v;
-				i=0;
-			}
-		}
-	}
-
-	*save = v;
-	*state = i;
-
-	/* quick scan back for '=' on the end somewhere */
-	/* fortunately we can drop 1 output char for each trailing = (upto 2) */
-	i=2;
-	while (inptr>in && i) {
-		inptr--;
-		if (_evc_base64_rank[*inptr] != 0xff) {
-			if (*inptr == '=' && outptr>out)
-				outptr--;
-			i--;
-		}
-	}
-
-	/* if i!= 0 then there is a truncation error! */
-	return outptr-out;
-}
-
-char *
-_evc_base64_encode_simple (const char *data, size_t len)
-{
-	unsigned char *out;
-	int state = 0, outlen;
-	int save = 0;
-
-	g_return_val_if_fail (data != NULL, NULL);
-
-	out = g_malloc (len * 4 / 3 + 5);
-	outlen = _evc_base64_encode_close ((unsigned char *)data, len, FALSE,
-				      out, &state, &save);
-	out[outlen] = '\0';
-	return (char *)out;
-}
-
-size_t
-_evc_base64_decode_simple (char *data, size_t len)
-{
-	int state = 0;
-	unsigned int save = 0;
-
-	g_return_val_if_fail (data != NULL, 0);
-
-	return _evc_base64_decode_step ((unsigned char *)data, len,
-					(unsigned char *)data, &state, &save);
-}
