@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -29,45 +29,45 @@
 #include <config.h>
 #endif
 
-#include <sys/types.h>
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/types.h>
 
 #include <glib.h>
+#include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 #include <gmodule.h>
 
-#include "libedataserver/e-msgport.h"
+#include <libedataserver/e-msgport.h>
 
 #include "camel-exception.h"
-#include "camel-i18n.h"
+#include "camel-private.h"
 #include "camel-provider.h"
 #include "camel-string-utils.h"
 #include "camel-vee-store.h"
-#include "camel-private.h"
 
 /* table of CamelProviderModule's */
 static GHashTable *module_table;
 /* table of CamelProvider's */
 static GHashTable *provider_table;
-static EMutex *provider_lock;
+static GStaticRecMutex provider_lock = G_STATIC_REC_MUTEX_INIT;
 
-#define LOCK() e_mutex_lock(provider_lock);
-#define UNLOCK() e_mutex_unlock(provider_lock);
+#define LOCK()		(g_static_rec_mutex_lock(&provider_lock))
+#define UNLOCK()	(g_static_rec_mutex_unlock(&provider_lock))
 
 /* The vfolder provider is always available */
 static CamelProvider vee_provider = {
 	"vfolder",
 	N_("Virtual folder email provider"),
-	
+
 	N_("For reading mail as a query of another set of folders"),
-	
+
 	"vfolder",
-	
+
 	CAMEL_PROVIDER_IS_STORAGE,
 	CAMEL_URL_NEED_PATH | CAMEL_URL_PATH_IS_ABSOLUTE | CAMEL_URL_FRAGMENT_IS_PATH,
-	
+
 	/* ... */
 };
 
@@ -76,7 +76,6 @@ static pthread_once_t setup_once = PTHREAD_ONCE_INIT;
 static void
 provider_setup(void)
 {
-	provider_lock = e_mutex_new(E_MUTEX_REC);
 	module_table = g_hash_table_new(camel_strcase_hash, camel_strcase_equal);
 	provider_table = g_hash_table_new(camel_strcase_hash, camel_strcase_equal);
 
@@ -122,14 +121,14 @@ camel_provider_init (void)
 			  CAMEL_PROVIDERDIR, g_strerror (errno));
 		return;
 	}
-	
+
 	while ((entry = g_dir_read_name (dir))) {
 		FILE *fp;
-		
+
 		p = strrchr (entry, '.');
 		if (!p || strcmp (p, ".urls") != 0)
 			continue;
-		
+
 		name = g_strdup_printf ("%s/%s", CAMEL_PROVIDERDIR, entry);
 		fp = g_fopen (name, "r");
 		if (!fp) {
@@ -138,7 +137,7 @@ camel_provider_init (void)
 			g_free (name);
 			continue;
 		}
-		
+
 		p = strrchr (name, '.');
 		strcpy (p, "." G_MODULE_SUFFIX);
 
@@ -150,7 +149,7 @@ camel_provider_init (void)
 			p = strchr (buf, '\n');
 			if (p)
 				*p = '\0';
-			
+
 			if (*buf) {
 				char *protocol = g_strdup(buf);
 
@@ -174,7 +173,7 @@ camel_provider_init (void)
  * Loads the provider at @path, and calls its initialization function,
  * passing @session as an argument. The provider should then register
  * itself with @session.
- **/ 
+ **/
 void
 camel_provider_load(const char *path, CamelException *ex)
 {
@@ -235,7 +234,7 @@ camel_provider_register(CamelProvider *provider)
 		UNLOCK();
 		return;
 	}
-	
+
 	for (i = 0; i < CAMEL_NUM_PROVIDER_TYPES; i++) {
 		if (provider->object_types[i])
 			provider->service_cache[i] = camel_object_bag_new (provider->url_hash, provider->url_equal,
